@@ -38,6 +38,11 @@
 #include <gtk/gtkentry.h>
 #include <gtk/gtktable.h>
 #include <gtk/gtkbutton.h>
+#include <gtk/gtkstatusbar.h>
+#include <gtk/gtkspinbutton.h>
+#include <gtk/gtkvbox.h>
+#include <gtk/gtknotebook.h>
+#include <gtk/gtkoptionmenu.h>
 
 #include "addressbook.h"
 #include "prefs_common.h"
@@ -57,6 +62,7 @@
 static struct _LDAPEdit {
 	GtkWidget *window;
 	GtkWidget *notebook;
+	GtkWidget *hbbox;
 	GtkWidget *ok_btn;
 	GtkWidget *cancel_btn;
 	GtkWidget *statusbar;
@@ -258,6 +264,7 @@ static void addressbook_edit_ldap_dialog_create( gboolean *cancelled ) {
 
 	ldapedit.window     = window;
 	ldapedit.notebook   = notebook;
+	ldapedit.hbbox      = hbbox;
 	ldapedit.ok_btn     = ok_btn;
 	ldapedit.cancel_btn = cancel_btn;
 	ldapedit.statusbar  = statusbar;
@@ -416,6 +423,7 @@ void addressbook_edit_ldap_page_extended( gint pageNum, gchar *pageLbl ) {
 
 	entry_bindPW = gtk_entry_new();
 	gtk_table_attach(GTK_TABLE(table), entry_bindPW, 1, 2, top, (top + 1), GTK_EXPAND|GTK_SHRINK|GTK_FILL, 0, 0, 0);
+	gtk_entry_set_visibility(GTK_ENTRY(entry_bindPW), FALSE);
 
 	/* Next row */
 	++top;
@@ -508,6 +516,8 @@ AdapterDSource *addressbook_edit_ldap( AddressIndex *addrIndex, AdapterDSource *
 
 	if (!ldapedit.window)
 		addressbook_edit_ldap_create(&cancelled);
+	gtkut_box_set_reverse_order(GTK_BOX(ldapedit.hbbox),
+				    !prefs_common.comply_gnome_hig);
 	gtk_notebook_set_current_page( GTK_NOTEBOOK(ldapedit.notebook), 0 );
 	gtk_widget_grab_focus(ldapedit.ok_btn);
 	gtk_widget_grab_focus(ldapedit.entry_name);
