@@ -26,12 +26,19 @@
 #include <gdk/gdkkeysyms.h>
 #include <gdk/gdk.h>
 #include <gtk/gtkwidget.h>
+#include <gtk/gtkbox.h>
 #include <gtk/gtkhbbox.h>
 #include <gtk/gtkbutton.h>
+#include <gtk/gtkarrow.h>
 #include <gtk/gtkctree.h>
 #include <gtk/gtkcombo.h>
 #include <gtk/gtkbindings.h>
 #include <gtk/gtkitemfactory.h>
+#include <gtk/gtktreemodel.h>
+#include <gtk/gtktreesortable.h>
+#include <gtk/gtktreeview.h>
+#include <gtk/gtktreestore.h>
+#include <gtk/gtkversion.h>
 #include <stdlib.h>
 #include <stdarg.h>
 
@@ -42,11 +49,9 @@
 
 #include "gtkutils.h"
 #include "utils.h"
-#include "gtksctree.h"
 #include "codeconv.h"
 #include "menu.h"
 
-#warning FIXME_GTK2
 gboolean gtkut_get_font_size(GtkWidget *widget, gint *width, gint *height)
 {
 	PangoLayout *layout;
@@ -106,6 +111,13 @@ void gtkut_convert_int_to_gdk_color(gint rgbvalue, GdkColor *color)
 	color->blue  = (int) (((gdouble) (rgbvalue & 0x0000ff)        / 255.0) * 65535.0);
 }
 
+static gboolean reverse_order = FALSE;
+
+void gtkut_stock_button_set_set_reverse(gboolean reverse)
+{
+	reverse_order = reverse;
+}
+
 void gtkut_stock_button_set_create(GtkWidget **bbox,
 				   GtkWidget **button1, const gchar *label1,
 				   GtkWidget **button2, const gchar *label2,
@@ -116,26 +128,58 @@ void gtkut_stock_button_set_create(GtkWidget **bbox,
 
 	*bbox = gtk_hbutton_box_new();
 	gtk_button_box_set_layout(GTK_BUTTON_BOX(*bbox), GTK_BUTTONBOX_END);
-	gtk_box_set_spacing(GTK_BOX(*bbox), 5);
-
-	*button1 = gtk_button_new_from_stock(label1);
-	GTK_WIDGET_SET_FLAGS(*button1, GTK_CAN_DEFAULT);
-	gtk_box_pack_start(GTK_BOX(*bbox), *button1, TRUE, TRUE, 0);
-	gtk_widget_show(*button1);
-
-	if (button2) {
-		*button2 = gtk_button_new_from_stock(label2);
-		GTK_WIDGET_SET_FLAGS(*button2, GTK_CAN_DEFAULT);
-		gtk_box_pack_start(GTK_BOX(*bbox), *button2, TRUE, TRUE, 0);
-		gtk_widget_show(*button2);
-	}
+	gtk_box_set_spacing(GTK_BOX(*bbox), 6);
 
 	if (button3) {
 		*button3 = gtk_button_new_from_stock(label3);
 		GTK_WIDGET_SET_FLAGS(*button3, GTK_CAN_DEFAULT);
-		gtk_box_pack_start(GTK_BOX(*bbox), *button3, TRUE, TRUE, 0);
+		gtk_box_pack_start(GTK_BOX(*bbox), *button3, FALSE, FALSE, 0);
 		gtk_widget_show(*button3);
 	}
+
+	if (button2) {
+		*button2 = gtk_button_new_from_stock(label2);
+		GTK_WIDGET_SET_FLAGS(*button2, GTK_CAN_DEFAULT);
+		gtk_box_pack_start(GTK_BOX(*bbox), *button2, FALSE, FALSE, 0);
+		gtk_widget_show(*button2);
+	}
+
+	*button1 = gtk_button_new_from_stock(label1);
+	GTK_WIDGET_SET_FLAGS(*button1, GTK_CAN_DEFAULT);
+	gtk_box_pack_start(GTK_BOX(*bbox), *button1, FALSE, FALSE, 0);
+	gtk_widget_show(*button1);
+
+	if (reverse_order)
+		gtkut_box_set_reverse_order(GTK_BOX(*bbox), TRUE);
+}
+
+void gtkut_box_set_reverse_order(GtkBox *box, gboolean reverse)
+{
+	GList *cur;
+	GList *new_order = NULL;
+	gint pos = 0;
+	gboolean is_reversed;
+
+	g_return_if_fail(box != NULL);
+
+	is_reversed = GPOINTER_TO_INT
+		(g_object_get_data(G_OBJECT(box), "reverse-order"));
+	if (is_reversed == reverse)
+		return;
+	g_object_set_data(G_OBJECT(box), "reverse-order",
+			  GINT_TO_POINTER(reverse));
+
+	for (cur = box->children; cur != NULL; cur = cur->next) {
+		GtkBoxChild *cinfo = cur->data;
+		new_order = g_list_prepend(new_order, cinfo->widget);
+	}
+
+	for (cur = new_order; cur != NULL; cur = cur->next) {
+		GtkWidget *child = cur->data;
+		gtk_box_reorder_child(box, child, pos++);
+	}
+
+	g_list_free(new_order);
 }
 
 static void combo_button_size_request(GtkWidget *widget,
@@ -239,54 +283,6 @@ ComboButton *gtkut_combo_button_create(GtkWidget *button,
 	return combo;
 }
 
-#define CELL_SPACING 1
-#define ROW_TOP_YPIXEL(clist, row) (((clist)->row_height * (row)) + \
-				    (((row) + 1) * CELL_SPACING) + \
-				    (clist)->voffset)
-#define ROW_FROM_YPIXEL(clist, y) (((y) - (clist)->voffset) / \
-				   ((clist)->row_height + CELL_SPACING))
-
-void gtkut_ctree_node_move_if_on_the_edge(GtkCTree *ctree, GtkCTreeNode *node)
-{
-	GtkCList *clist = GTK_CLIST(ctree);
-	gint row;
-	GtkVisibility row_visibility, prev_row_visibility, next_row_visibility;
-
-	g_return_if_fail(ctree != NULL);
-	g_return_if_fail(node != NULL);
-
-	row = g_list_position(clist->row_list, (GList *)node);
-	if (row < 0 || row >= clist->rows || clist->row_height == 0) return;
-	row_visibility = gtk_clist_row_is_visible(clist, row);
-	prev_row_visibility = gtk_clist_row_is_visible(clist, row - 1);
-	next_row_visibility = gtk_clist_row_is_visible(clist, row + 1);
-
-	if (row_visibility == GTK_VISIBILITY_NONE) {
-		gtk_clist_moveto(clist, row, -1, 0.5, 0);
-		return;
-	}
-	if (row_visibility == GTK_VISIBILITY_FULL &&
-	    prev_row_visibility == GTK_VISIBILITY_FULL &&
-	    next_row_visibility == GTK_VISIBILITY_FULL)
-		return;
-	if (prev_row_visibility != GTK_VISIBILITY_FULL &&
-	    next_row_visibility != GTK_VISIBILITY_FULL)
-		return;
-
-	if (prev_row_visibility != GTK_VISIBILITY_FULL) {
-		gtk_clist_moveto(clist, row, -1, 0.2, 0);
-		return;
-	}
-	if (next_row_visibility != GTK_VISIBILITY_FULL) {
-		gtk_clist_moveto(clist, row, -1, 0.8, 0);
-		return;
-	}
-}
-
-#undef CELL_SPACING
-#undef ROW_TOP_YPIXEL
-#undef ROW_FROM_YPIXEL
-
 gint gtkut_ctree_get_nth_from_node(GtkCTree *ctree, GtkCTreeNode *node)
 {
 	g_return_val_if_fail(ctree != NULL, -1);
@@ -295,82 +291,6 @@ gint gtkut_ctree_get_nth_from_node(GtkCTree *ctree, GtkCTreeNode *node)
 	return g_list_position(GTK_CLIST(ctree)->row_list, (GList *)node);
 }
 
-/* get the next node, including the invisible one */
-GtkCTreeNode *gtkut_ctree_node_next(GtkCTree *ctree, GtkCTreeNode *node)
-{
-	GtkCTreeNode *parent;
-
-	if (!node) return NULL;
-
-	if (GTK_CTREE_ROW(node)->children)
-		return GTK_CTREE_ROW(node)->children;
-
-	if (GTK_CTREE_ROW(node)->sibling)
-		return GTK_CTREE_ROW(node)->sibling;
-
-	for (parent = GTK_CTREE_ROW(node)->parent; parent != NULL;
-	     parent = GTK_CTREE_ROW(parent)->parent) {
-		if (GTK_CTREE_ROW(parent)->sibling)
-			return GTK_CTREE_ROW(parent)->sibling;
-	}
-
-	return NULL;
-}
-
-/* get the previous node, including the invisible one */
-GtkCTreeNode *gtkut_ctree_node_prev(GtkCTree *ctree, GtkCTreeNode *node)
-{
-	GtkCTreeNode *prev;
-	GtkCTreeNode *child;
-
-	if (!node) return NULL;
-
-	prev = GTK_CTREE_NODE_PREV(node);
-	if (prev == GTK_CTREE_ROW(node)->parent)
-		return prev;
-
-	child = prev;
-	while (GTK_CTREE_ROW(child)->children != NULL) {
-		child = GTK_CTREE_ROW(child)->children;
-		while (GTK_CTREE_ROW(child)->sibling != NULL)
-			child = GTK_CTREE_ROW(child)->sibling;
-	}
-
-	return child;
-}
-
-gboolean gtkut_ctree_node_is_selected(GtkCTree *ctree, GtkCTreeNode *node)
-{
-	GtkCList *clist = GTK_CLIST(ctree);
-	GList *cur;
-
-	for (cur = clist->selection; cur != NULL; cur = cur->next) {
-		if (node == GTK_CTREE_NODE(cur->data))
-			return TRUE;
-	}
-
-	return FALSE;
-}
-
-GtkCTreeNode *gtkut_ctree_find_collapsed_parent(GtkCTree *ctree,
-						GtkCTreeNode *node)
-{
-	if (!node) return NULL;
-
-	while ((node = GTK_CTREE_ROW(node)->parent) != NULL) {
-		if (!GTK_CTREE_ROW(node)->expanded)
-			return node;
-	}
-
-	return NULL;
-}
-
-void gtkut_ctree_expand_parent_all(GtkCTree *ctree, GtkCTreeNode *node)
-{
-	while ((node = gtkut_ctree_find_collapsed_parent(ctree, node)) != NULL)
-		gtk_ctree_expand(ctree, node);
-}
-
 void gtkut_ctree_set_focus_row(GtkCTree *ctree, GtkCTreeNode *node)
 {
 	gtkut_clist_set_focus_row(GTK_CLIST(ctree),
@@ -415,6 +335,60 @@ gboolean gtkut_tree_model_next(GtkTreeModel *model, GtkTreeIter *iter)
 	return FALSE;
 }
 
+gboolean gtkut_tree_model_prev(GtkTreeModel *model, GtkTreeIter *iter)
+{
+	GtkTreeIter iter_, child, next, parent;
+	GtkTreePath *path;
+	gboolean found = FALSE;
+
+	iter_ = *iter;
+
+	path = gtk_tree_model_get_path(model, &iter_);
+
+	if (gtk_tree_path_prev(path)) {
+		gtk_tree_model_get_iter(model, &child, path);
+
+		while (gtk_tree_model_iter_has_child(model, &child)) {
+			iter_ = child;
+			gtk_tree_model_iter_children(model, &child, &iter_);
+			next = child;
+			while (gtk_tree_model_iter_next(model, &next))
+				child = next;
+		}
+
+		*iter = child;
+		found = TRUE;
+	} else if (gtk_tree_model_iter_parent(model, &parent, &iter_)) {
+		*iter = parent;
+		found = TRUE;
+	}
+
+	gtk_tree_path_free(path);
+
+	return found;
+}
+
+gboolean gtkut_tree_model_get_iter_last(GtkTreeModel *model, GtkTreeIter *iter)
+{
+	GtkTreeIter iter_, child, next;
+
+	if (!gtk_tree_model_get_iter_first(model, &iter_))
+		return FALSE;
+
+	for (;;) {
+		next = iter_;
+		while (gtk_tree_model_iter_next(model, &next))
+			iter_ = next;
+		if (gtk_tree_model_iter_children(model, &child, &iter_))
+			iter_ = child;
+		else
+			break;
+	}
+
+	*iter = iter_;
+	return TRUE;
+}
+
 gboolean gtkut_tree_model_find_by_column_data(GtkTreeModel *model,
 					      GtkTreeIter *iter,
 					      GtkTreeIter *start,
@@ -424,41 +398,63 @@ gboolean gtkut_tree_model_find_by_column_data(GtkTreeModel *model,
 	GtkTreeIter iter_;
 	gpointer store_data;
 
-	if (start)
+	if (start) {
+		gtk_tree_model_get(model, start, col, &store_data, -1);
+		if (store_data == data) {
+			*iter = *start;
+			return TRUE;
+		}
 		valid = gtk_tree_model_iter_children(model, &iter_, start);
-	else
+	} else
 		valid = gtk_tree_model_get_iter_first(model, &iter_);
 
 	while (valid) {
-		gtk_tree_model_get(model, &iter_, col, &store_data, -1);
-		if (store_data == data) {
-			*iter = iter_;
+		if (gtkut_tree_model_find_by_column_data
+			(model, iter, &iter_, col, data)) {
 			return TRUE;
 		}
 
-		if (gtk_tree_model_iter_has_child(model, &iter_)) {
-			if (gtkut_tree_model_find_by_column_data
-				(model, iter, &iter_, col, data)) {
-				return TRUE;
-			}
-		}
-
 		valid = gtk_tree_model_iter_next(model, &iter_);
 	}
 
 	return FALSE;
 }
 
+gboolean gtkut_tree_row_reference_get_iter(GtkTreeModel *model,
+					   GtkTreeRowReference *ref,
+					   GtkTreeIter *iter)
+{
+	GtkTreePath *path;
+	gboolean valid = FALSE;
+
+	if (ref) {
+		path = gtk_tree_row_reference_get_path(ref);
+		if (path) {
+			valid = gtk_tree_model_get_iter(model, iter, path);
+			gtk_tree_path_free(path);
+		}
+	}
+
+	return valid;
+}
+
 gboolean gtkut_tree_row_reference_equal(GtkTreeRowReference *ref1,
 					GtkTreeRowReference *ref2)
 {
 	GtkTreePath *path1, *path2;
 	gint result;
 
-	g_return_val_if_fail(ref1 != NULL && ref2 != NULL, FALSE);
+	if (ref1 == NULL || ref2 == NULL)
+		return FALSE;
 
 	path1 = gtk_tree_row_reference_get_path(ref1);
+	if (!path1)
+		return FALSE;
 	path2 = gtk_tree_row_reference_get_path(ref2);
+	if (!path2) {
+		gtk_tree_path_free(path1);
+		return FALSE;
+	}
 
 	result = gtk_tree_path_compare(path1, path2);
 
@@ -468,6 +464,27 @@ gboolean gtkut_tree_row_reference_equal(GtkTreeRowReference *ref1,
 	return (result == 0);
 }
 
+void gtkut_tree_sortable_unset_sort_column_id(GtkTreeSortable *sortable)
+{
+#if GTK_CHECK_VERSION(2, 6, 0)
+	gtk_tree_sortable_set_sort_column_id
+		(sortable, GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID,
+		 GTK_SORT_ASCENDING);
+#else
+	GtkTreeStore *store = GTK_TREE_STORE(sortable);
+
+	g_return_if_fail(GTK_IS_TREE_STORE(sortable));
+
+	if (store->sort_column_id == -2 && store->order == GTK_SORT_ASCENDING)
+		return;
+
+	store->sort_column_id = -2;
+	store->order = GTK_SORT_ASCENDING;
+
+	gtk_tree_sortable_sort_column_changed(sortable);
+#endif
+}
+
 gboolean gtkut_tree_view_find_collapsed_parent(GtkTreeView *treeview,
 					       GtkTreeIter *parent,
 					       GtkTreeIter *iter)
@@ -497,6 +514,21 @@ gboolean gtkut_tree_view_find_collapsed_parent(GtkTreeView *treeview,
 	return FALSE;
 }
 
+void gtkut_tree_view_expand_parent_all(GtkTreeView *treeview, GtkTreeIter *iter)
+{
+	GtkTreeModel *model;
+	GtkTreeIter parent;
+	GtkTreePath *path;
+
+	model = gtk_tree_view_get_model(treeview);
+
+	if (gtk_tree_model_iter_parent(model, &parent, iter)) {
+		path = gtk_tree_model_get_path(model, &parent);
+		gtk_tree_view_expand_to_path(treeview, path);
+		gtk_tree_path_free(path);
+	}
+}
+
 #define SCROLL_EDGE_SIZE 15
 
 /* borrowed from gtktreeview.c */
@@ -527,6 +559,39 @@ void gtkut_tree_view_vertical_autoscroll(GtkTreeView *treeview)
 	gtk_adjustment_set_value(vadj, value);
 }
 
+/* modified version of gtk_tree_view_scroll_to_cell */
+void gtkut_tree_view_scroll_to_cell(GtkTreeView *treeview, GtkTreePath *path)
+{
+	GdkRectangle cell_rect;
+	GdkRectangle vis_rect;
+	gint dest_x, dest_y;
+	gint margin = 0;
+
+	if (!path)
+		return;
+
+	gtk_tree_view_get_cell_area(treeview, path, NULL, &cell_rect);
+	gtk_tree_view_widget_to_tree_coords(treeview, cell_rect.x, cell_rect.y,
+					    NULL, &(cell_rect.y));
+	gtk_tree_view_get_visible_rect(treeview, &vis_rect);
+
+	dest_x = vis_rect.x;
+	dest_y = vis_rect.y;
+
+	/* add margin */
+	if (cell_rect.height * 2 < vis_rect.height)
+		margin = cell_rect.height + 2;
+
+	if (cell_rect.y < vis_rect.y + margin)
+		dest_y = cell_rect.y - margin;
+	if (cell_rect.y + cell_rect.height >
+		vis_rect.y + vis_rect.height - margin)
+		dest_y = cell_rect.y + cell_rect.height - vis_rect.height +
+			margin;
+
+	gtk_tree_view_scroll_to_point(treeview, dest_x, dest_y);
+}
+
 void gtkut_combo_set_items(GtkCombo *combo, const gchar *str1, ...)
 {
 	va_list args;
@@ -792,13 +857,10 @@ void gtkut_widget_get_uposition(GtkWidget *widget, gint *px, gint *py)
 	*py = y;
 }
 
-#warning FIXME_GTK2
-void gtkut_widget_wait_for_draw(GtkWidget *widget)
+void gtkut_widget_draw_now(GtkWidget *widget)
 {
-	if (!GTK_WIDGET_VISIBLE(widget) || !GTK_WIDGET_MAPPED(widget)) return;
-
-	while (gtk_events_pending())
-		gtk_main_iteration();
+	if (GTK_WIDGET_VISIBLE(widget) && GTK_WIDGET_DRAWABLE(widget))
+		gdk_window_process_updates(widget->window, FALSE);
 }
 
 static void gtkut_clist_bindings_add(GtkWidget *clist)
@@ -832,10 +894,4 @@ void gtkut_widget_init(void)
 	gtk_object_sink(GTK_OBJECT(clist));
 	gtkut_clist_bindings_add(clist);
 	g_object_unref(G_OBJECT(clist));
-
-	clist = gtk_sctree_new_with_titles(1, 0, NULL);
-	g_object_ref(G_OBJECT(clist));
-	gtk_object_sink(GTK_OBJECT(clist));
-	gtkut_clist_bindings_add(clist);
-	g_object_unref(G_OBJECT(clist));
 }
