@@ -27,6 +27,8 @@
 #include <glib/gi18n.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkrc.h>
+#include <gtk/gtkstock.h>
+#include <gtk/gtkaccelmap.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -71,6 +73,7 @@
 #include "utils.h"
 #include "gtkutils.h"
 #include "socket.h"
+#include "stock_pixmap.h"
 
 #if USE_GPGME
 #  include "rfc2015.h"
@@ -143,6 +146,7 @@ int main(int argc, char *argv[])
 	gchar *userrc;
 	MainWindow *mainwin;
 	FolderView *folderview;
+	GdkPixbuf *icon;
 
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
@@ -210,6 +214,29 @@ int main(int argc, char *argv[])
 
 	/* migration from ~/.sylpheed to ~/.sylpheed-2.0 */
 	if (!is_dir_exist(RC_DIR)) {
+		const gchar *envstr;
+		AlertValue val;
+
+		/* check for filename encoding */
+		if (conv_get_locale_charset() != C_UTF_8) {
+			envstr = g_getenv("G_FILENAME_ENCODING");
+			if (!envstr)
+				envstr = g_getenv("G_BROKEN_FILENAMES");
+			if (!envstr) {
+				val = alertpanel(_("Filename encoding"),
+						 _("The locale encoding is not UTF-8, but the environmental variable G_FILENAME_ENCODING is not set.\n"
+						   "If the locale encoding is used for file name or directory name, it will not work correctly.\n"
+						   "In that case, you must set the following environmental variable (see README for detail):\n"
+						   "\n"
+						   "\tG_FILENAME_ENCODING=@locale\n"
+						   "\n"
+						   "Continue?"),
+						 GTK_STOCK_OK, GTK_STOCK_QUIT,
+						 NULL);
+				if (G_ALERTDEFAULT != val)
+					return 1;
+			}
+		}
 		if (make_dir(RC_DIR) < 0)
 			return 1;
 		if (is_dir_exist(OLD_RC_DIR))
@@ -222,6 +249,10 @@ int main(int argc, char *argv[])
 	MAKE_DIR_IF_NOT_EXIST(get_tmp_dir());
 	MAKE_DIR_IF_NOT_EXIST(RC_DIR G_DIR_SEPARATOR_S UIDL_DIR);
 
+	/* remove temporary files */
+	remove_all_files(get_tmp_dir());
+	remove_all_files(get_mime_tmp_dir());
+
 	if (is_file_exist(RC_DIR G_DIR_SEPARATOR_S "sylpheed.log")) {
 		if (rename(RC_DIR G_DIR_SEPARATOR_S "sylpheed.log",
 			   RC_DIR G_DIR_SEPARATOR_S "sylpheed.log.bak") < 0)
@@ -232,22 +263,26 @@ int main(int argc, char *argv[])
 	prefs_common_read_config();
 
 #if USE_GPGME
-	if (gpgme_check_version("0.4.5")) {  /* Also does some gpgme init */
-        gpgme_engine_info_t engineInfo;
+	if (gpgme_check_version("0.4.5")) {
+		/* Also does some gpgme init */
+	        gpgme_engine_info_t engineInfo;
+
 		rfc2015_disable_all();
 
-        gpgme_set_locale(NULL, LC_CTYPE, setlocale(LC_CTYPE, NULL));
-        gpgme_set_locale(NULL, LC_MESSAGES, setlocale(LC_MESSAGES, NULL));
+		gpgme_set_locale(NULL, LC_CTYPE, setlocale(LC_CTYPE, NULL));
+		gpgme_set_locale(NULL, LC_MESSAGES,
+				 setlocale(LC_MESSAGES, NULL));
 
-        if (!gpgme_get_engine_info(&engineInfo)) {
-            while (engineInfo) {
-                debug_print("GpgME Protocol: %s\n      Version: %s\n",
-                    gpgme_get_protocol_name(engineInfo->protocol),
-                    engineInfo->version);
-                engineInfo = engineInfo->next;
-            }
-        }
-    } else {
+		if (!gpgme_get_engine_info(&engineInfo)) {
+			while (engineInfo) {
+				debug_print("GpgME Protocol: %s\n      Version: %s\n",
+					    gpgme_get_protocol_name
+						(engineInfo->protocol),
+					    engineInfo->version);
+				engineInfo = engineInfo->next;
+			}
+		}
+	} else {
 		if (prefs_common.gpg_warning) {
 			AlertValue val;
 
@@ -260,12 +295,12 @@ int main(int argc, char *argv[])
 				prefs_common.gpg_warning = FALSE;
 		}
 	}
-    /* FIXME: This function went away.  We can either block until gpgme
-     * operations finish (currently implemented) or register callbacks
-     * with the gtk main loop via the gpgme io callback interface instead.
-     *
+	/* FIXME: This function went away.  We can either block until gpgme
+	 * operations finish (currently implemented) or register callbacks
+	 * with the gtk main loop via the gpgme io callback interface instead.
+	 *
 	 * gpgme_register_idle(idle_function_for_gpgme);
-     */
+	 */
 #endif
 
 	sock_set_io_timeout(prefs_common.io_timeout_secs);
@@ -275,6 +310,8 @@ int main(int argc, char *argv[])
 	prefs_display_header_read_config();
 
 	gtkut_widget_init();
+	stock_pixbuf_gdk(NULL, STOCK_PIXMAP_SYLPHEED, &icon);
+	gtk_window_set_default_icon(icon);
 
 	mainwin = main_window_create
 		(prefs_common.sep_folder | prefs_common.sep_msg << 1);
@@ -492,7 +529,8 @@ void app_will_exit(GtkWidget *widget, gpointer data)
 	gtk_accel_map_save(filename);
 	g_free(filename);
 
-	/* delete temporary files */
+	/* remove temporary files */
+	remove_all_files(get_tmp_dir());
 	remove_all_files(get_mime_tmp_dir());
 
 	close_log_file();
@@ -807,7 +845,8 @@ static void send_queue(void)
 			gint ret;
 
 			ret = procmsg_send_queue(folder->queue,
-						 prefs_common.savemsg);
+						 prefs_common.savemsg,
+						 prefs_common.filter_sent);
 			statusbar_pop_all();
 			if (ret > 0)
 				folder_item_scan(folder->queue);
