@@ -38,6 +38,7 @@
 #include "prefs.h"
 #include "prefs_account.h"
 #include "prefs_customheader.h"
+#include "prefs_common.h"
 #include "account.h"
 #include "mainwindow.h"
 #include "manage_window.h"
@@ -96,6 +97,10 @@ static struct Receive {
 	GtkWidget *imap_frame;
 	GtkWidget *imap_auth_type_optmenu;
 
+	GtkWidget *nntp_frame;
+	GtkWidget *maxarticle_spinbtn;
+	GtkObject *maxarticle_spinbtn_adj;
+
 	GtkWidget *recvatgetall_chkbtn;
 } receive;
 
@@ -126,8 +131,9 @@ static struct Compose {
 
 #if USE_GPGME
 static struct Privacy {
-	GtkWidget *default_encrypt_chkbtn;
 	GtkWidget *default_sign_chkbtn;
+	GtkWidget *default_encrypt_chkbtn;
+	GtkWidget *encrypt_reply_chkbtn;
 	GtkWidget *ascii_armored_chkbtn;
 	GtkWidget *clearsign_chkbtn;
 	GtkWidget *defaultkey_radiobtn;
@@ -284,6 +290,10 @@ static PrefParam param[] = {
 	 prefs_account_imap_auth_type_set_data_from_optmenu,
 	 prefs_account_imap_auth_type_set_optmenu},
 
+	{"max_nntp_articles", "300", &tmp_ac_prefs.max_nntp_articles, P_INT,
+	 &receive.maxarticle_spinbtn,
+	 prefs_set_data_from_spinbtn, prefs_set_spinbtn},
+
 	{"receive_at_get_all", "TRUE", &tmp_ac_prefs.recv_at_getall, P_BOOL,
 	 &receive.recvatgetall_chkbtn,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
@@ -357,11 +367,14 @@ static PrefParam param[] = {
 
 #if USE_GPGME
 	/* Privacy */
+	{"default_sign", "FALSE", &tmp_ac_prefs.default_sign, P_BOOL,
+	 &privacy.default_sign_chkbtn,
+	 prefs_set_data_from_toggle, prefs_set_toggle},
 	{"default_encrypt", "FALSE", &tmp_ac_prefs.default_encrypt, P_BOOL,
 	 &privacy.default_encrypt_chkbtn,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
-	{"default_sign", "FALSE", &tmp_ac_prefs.default_sign, P_BOOL,
-	 &privacy.default_sign_chkbtn,
+	{"encrypt_reply", "TRUE", &tmp_ac_prefs.encrypt_reply, P_BOOL,
+	 &privacy.encrypt_reply_chkbtn,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
 	{"ascii_armored", "FALSE", &tmp_ac_prefs.ascii_armored, P_BOOL,
 	 &privacy.ascii_armored_chkbtn,
@@ -620,6 +633,8 @@ PrefsAccount *prefs_account_open(PrefsAccount *ac_prefs)
 		prefs_account_create();
 	}
 
+	gtkut_box_set_reverse_order(GTK_BOX(dialog.confirm_area),
+				    !prefs_common.comply_gnome_hig);
 	manage_window_set_transient(GTK_WINDOW(dialog.window));
 	gtk_notebook_set_current_page(GTK_NOTEBOOK(dialog.notebook), 0);
 	gtk_widget_grab_focus(dialog.ok_btn);
@@ -976,10 +991,18 @@ static void prefs_account_receive_create(void)
 	GtkWidget *inbox_label;
 	GtkWidget *inbox_entry;
 	GtkWidget *inbox_btn;
+
 	GtkWidget *imap_frame;
 	GtkWidget *optmenu;
 	GtkWidget *optmenu_menu;
 	GtkWidget *menuitem;
+
+	GtkWidget *nntp_frame;
+	GtkWidget *maxarticle_label;
+	GtkWidget *maxarticle_spinbtn;
+	GtkObject *maxarticle_spinbtn_adj;
+	GtkWidget *maxarticle_desc_label;
+
 	GtkWidget *recvatgetall_chkbtn;
 
 	vbox1 = gtk_vbox_new (FALSE, VSPACING);
@@ -1088,16 +1111,9 @@ static void prefs_account_receive_create(void)
 
 	PACK_VSPACER(vbox2, vbox3, VSPACING_NARROW_2);
 
-	hbox1 = gtk_hbox_new (FALSE, 8);
-	gtk_widget_show (hbox1);
-	gtk_box_pack_start (GTK_BOX (vbox2), hbox1, FALSE, FALSE, 0);
-
-	label = gtk_label_new
-		(_("(Unfiltered messages will be stored in this folder)"));
-	gtk_widget_show (label);
-	gtk_box_pack_start (GTK_BOX (hbox1), label, FALSE, FALSE, 0);
-	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
-	gtkut_widget_set_small_font_size (label);
+	PACK_SMALL_LABEL
+		(vbox2, label, 
+		 _("Unfiltered messages will be stored in this folder."));
 
 	PACK_FRAME (vbox1, imap_frame, _("IMAP4"));
 
@@ -1126,6 +1142,37 @@ static void prefs_account_receive_create(void)
 
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (optmenu), optmenu_menu);
 
+	PACK_FRAME (vbox1, nntp_frame, _("News"));
+
+	vbox2 = gtk_vbox_new (FALSE, 0);
+	gtk_widget_show (vbox2);
+	gtk_container_add (GTK_CONTAINER (nntp_frame), vbox2);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox2), 8);
+
+	hbox1 = gtk_hbox_new (FALSE, 8);
+	gtk_widget_show (hbox1);
+	gtk_box_pack_start (GTK_BOX (vbox2), hbox1, FALSE, FALSE, 0);
+
+	maxarticle_label = gtk_label_new
+		(_("Maximum number of articles to download"));
+	gtk_widget_show (maxarticle_label);
+	gtk_box_pack_start (GTK_BOX (hbox1), maxarticle_label, FALSE, FALSE, 0);
+	gtk_label_set_justify (GTK_LABEL (maxarticle_label), GTK_JUSTIFY_LEFT);
+
+	maxarticle_spinbtn_adj =
+		gtk_adjustment_new (300, 0, 10000, 10, 100, 100);
+	maxarticle_spinbtn = gtk_spin_button_new
+		(GTK_ADJUSTMENT (maxarticle_spinbtn_adj), 10, 0);
+	gtk_widget_show (maxarticle_spinbtn);
+	gtk_box_pack_start (GTK_BOX (hbox1), maxarticle_spinbtn,
+			    FALSE, FALSE, 0);
+	gtk_widget_set_size_request (maxarticle_spinbtn, 64, -1);
+	gtk_spin_button_set_numeric
+		(GTK_SPIN_BUTTON (maxarticle_spinbtn), TRUE);
+
+	PACK_SMALL_LABEL (vbox2, maxarticle_desc_label,
+			  _("No limit if 0 is specified."));
+
 	PACK_CHECK_BUTTON
 		(vbox1, recvatgetall_chkbtn,
 		 _("`Get all' checks for new messages on this account"));
@@ -1145,6 +1192,10 @@ static void prefs_account_receive_create(void)
 	receive.imap_frame             = imap_frame;
 	receive.imap_auth_type_optmenu = optmenu;
 
+	receive.nntp_frame             = nntp_frame;
+	receive.maxarticle_spinbtn     = maxarticle_spinbtn;
+	receive.maxarticle_spinbtn_adj = maxarticle_spinbtn_adj;
+
 	receive.recvatgetall_chkbtn = recvatgetall_chkbtn;
 }
 
@@ -1434,8 +1485,9 @@ static void prefs_account_privacy_create(void)
 	GtkWidget *vbox2;
 	GtkWidget *hbox1;
 	GtkWidget *label;
-	GtkWidget *default_encrypt_chkbtn;
 	GtkWidget *default_sign_chkbtn;
+	GtkWidget *default_encrypt_chkbtn;
+	GtkWidget *encrypt_reply_chkbtn;
 	GtkWidget *ascii_armored_chkbtn;
 	GtkWidget *clearsign_chkbtn;
 	GtkWidget *defaultkey_radiobtn;
@@ -1452,10 +1504,12 @@ static void prefs_account_privacy_create(void)
 	gtk_widget_show (vbox2);
 	gtk_box_pack_start (GTK_BOX (vbox1), vbox2, FALSE, FALSE, 0);
 
-	PACK_CHECK_BUTTON (vbox2, default_encrypt_chkbtn,
-			   _("Encrypt message by default"));
 	PACK_CHECK_BUTTON (vbox2, default_sign_chkbtn,
 			   _("Sign message by default"));
+	PACK_CHECK_BUTTON (vbox2, default_encrypt_chkbtn,
+			   _("Encrypt message by default"));
+	PACK_CHECK_BUTTON (vbox2, encrypt_reply_chkbtn,
+			   _("Encrypt when replying to encrypted message"));
 	PACK_CHECK_BUTTON (vbox2, ascii_armored_chkbtn,
 			   _("Use ASCII-armored format for encryption"));
 	PACK_CHECK_BUTTON (vbox2, clearsign_chkbtn,
@@ -1517,8 +1571,9 @@ static void prefs_account_privacy_create(void)
 
 	SET_TOGGLE_SENSITIVITY (customkey_radiobtn, customkey_entry);
 
-	privacy.default_encrypt_chkbtn = default_encrypt_chkbtn;
 	privacy.default_sign_chkbtn    = default_sign_chkbtn;
+	privacy.default_encrypt_chkbtn = default_encrypt_chkbtn;
+	privacy.encrypt_reply_chkbtn   = encrypt_reply_chkbtn;
 	privacy.ascii_armored_chkbtn   = ascii_armored_chkbtn;
 	privacy.clearsign_chkbtn       = clearsign_chkbtn;
 	privacy.defaultkey_radiobtn    = defaultkey_radiobtn;
@@ -1584,8 +1639,6 @@ static void prefs_account_ssl_create(void)
 
 	GtkWidget *vbox6;
 	GtkWidget *use_nonblocking_ssl_chkbtn;
-	GtkWidget *hbox;
-	GtkWidget *hbox_spc;
 	GtkWidget *label;
 
 	vbox1 = gtk_vbox_new (FALSE, VSPACING);
@@ -1667,21 +1720,9 @@ static void prefs_account_ssl_create(void)
 
 	PACK_CHECK_BUTTON(vbox6, use_nonblocking_ssl_chkbtn,
 			  _("Use non-blocking SSL"));
-
-	hbox = gtk_hbox_new (FALSE, 0);
-	gtk_widget_show (hbox);
-	gtk_box_pack_start (GTK_BOX (vbox6), hbox, FALSE, FALSE, 0);
-
-	hbox_spc = gtk_hbox_new (FALSE, 0);
-	gtk_widget_show (hbox_spc);
-	gtk_box_pack_start (GTK_BOX (hbox), hbox_spc, FALSE, FALSE, 0);
-	gtk_widget_set_size_request (hbox_spc, 16, -1);
-
-	label = gtk_label_new
-		(_("(Turn this off if you have problems in SSL connection)"));
-	gtk_widget_show (label);
-	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-	gtkut_widget_set_small_font_size (label);
+	PACK_SMALL_LABEL
+		(vbox6, label,
+		 _("Turn this off if you have problems in SSL connection."));
 
 	ssl.pop_frame               = pop_frame;
 	ssl.pop_nossl_radiobtn      = pop_nossl_radiobtn;
@@ -2178,6 +2219,7 @@ static void prefs_account_protocol_activated(GtkMenuItem *menuitem)
 		gtk_widget_set_sensitive(basic.pass_entry, active);
 		gtk_widget_hide(receive.pop3_frame);
 		gtk_widget_hide(receive.imap_frame);
+		gtk_widget_show(receive.nntp_frame);
 		gtk_widget_set_sensitive(receive.recvatgetall_chkbtn, TRUE);
 
 		if (!tmp_ac_prefs.account_name) {
@@ -2213,6 +2255,7 @@ static void prefs_account_protocol_activated(GtkMenuItem *menuitem)
 		gtk_widget_set_sensitive(basic.pass_entry, FALSE);
 		gtk_widget_hide(receive.pop3_frame);
 		gtk_widget_hide(receive.imap_frame);
+		gtk_widget_hide(receive.nntp_frame);
 		gtk_widget_set_sensitive(receive.recvatgetall_chkbtn, FALSE);
 
 		if (!tmp_ac_prefs.account_name) {
@@ -2248,6 +2291,7 @@ static void prefs_account_protocol_activated(GtkMenuItem *menuitem)
 		gtk_widget_set_sensitive(basic.pass_entry, TRUE);
 		gtk_widget_hide(receive.pop3_frame);
 		gtk_widget_show(receive.imap_frame);
+		gtk_widget_hide(receive.nntp_frame);
 		gtk_widget_set_sensitive(receive.recvatgetall_chkbtn, TRUE);
 
 		if (!tmp_ac_prefs.account_name) {
@@ -2284,6 +2328,7 @@ static void prefs_account_protocol_activated(GtkMenuItem *menuitem)
 		gtk_widget_set_sensitive(basic.pass_entry, TRUE);
 		gtk_widget_show(receive.pop3_frame);
 		gtk_widget_hide(receive.imap_frame);
+		gtk_widget_hide(receive.nntp_frame);
 		gtk_widget_set_sensitive(receive.recvatgetall_chkbtn, TRUE);
 
 		if (!tmp_ac_prefs.account_name) {
